/***************************************************************************
                          sglwindow.cpp  -  description
                             -------------------
    begin                : Wed Jan 23 2002
    copyright            : (C) 2002 by Andre Stein
    email                : andre_stein@web.de
    url                  : www.steinsoft.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sglwindow.h"
#include <SDL.h>

const float MOVE_DECELERATE = 0.001f;

SGLWindow::SGLWindow()
{
	camera[0] = 0.0f;
	camera[1] = -1.5f;
	camera[2] = -14.0f;

	moving[0] = 0.0f;
	moving[1] = 0.0f;
}
SGLWindow::~SGLWindow()
{
}
/** No descriptions */
void SGLWindow::init()
{
	glClearColor(0.0f,0.0f,0.0f,0.0f);
	glShadeModel(GL_SMOOTH);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	//enable texture mapping
	glEnable(GL_TEXTURE_2D);

	/*
	 * create 2 textures from the same bitmap
	 *  one normally, one with anisotropy filtering
	 */

	filteredTexture.createMipmap("data/texture.bmp", GL_LINEAR_MIPMAP_NEAREST, GL_LINEAR,
					2.0f);
	normalTexture.createMipmap("data/texture.bmp");

	// set title
	SDL_WM_SetCaption("{steinsoft.net} - anisotropy test",0);
}
/** No descriptions */
void SGLWindow::resize(int width, int height)
{
	glViewport(0,0,width,height);

	if (height == 0)
		height = 1;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(45.0f,width/height,0.1f,100.0f);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}
/** No descriptions */
void SGLWindow::update(float frametime)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();

	//translate depending on camera position
	glTranslatef(camera[0], camera[1], camera[2]);

	/**
	 * draw things
	 ****/

	// non-filtered
	glTranslatef(-1.5,0.0f,0.0f);
	normalTexture.select();
	drawObject();

	// filtered
	glTranslatef(3.0f,0.0f,0.0f);
	// flip
	glScalef(-1.0f,1.0f,1.0f);
	filteredTexture.select();
	drawObject();

	/**
	 * other things
	 ****/

	//update mouse etc.
	checkMouse();

	// update movements
	//  seems complicated butn creates a cool effect
	//  with a dynamic movement
	camera[0] += moving[0] * frametime;
	camera[2] += moving[1] * frametime;

	//new movement speeds..
	moving[0] /= 1.001f;
	moving[1] /= 1.001f;
}

void SGLWindow::keyDown(int key)
{
	switch(key)
 	{
    	case SDLK_ESCAPE:
     		SDL_Event sent;
       		sent.type = SDL_QUIT;

         	SDL_PushEvent(&sent);
          	break;
        }
}

void SGLWindow::checkMouse()
{
	int x,
	    y;

	//get mouse state
	int button = SDL_GetMouseState(&x, &y);	

	//calculate mouse movement
        float moveX = (320 - x)/10000.0f;
        float moveY = (240 - y)/10000.0f;

	/*
         Now do movements depending on mouse button pressed
         */

	if (moveX < 0.0f)
		moving[0] += moveX;
	else if (moveX > 0.0f)
		moving[0] += moveX;

	if (moveY < 0.0f)
		moving[1] += moveY;
	else if (moveY > 0.0f)
		moving[1] += moveY;

	//move cursor to initial position
	SDL_WarpMouse(320,240);
}

void SGLWindow::drawObject()
{
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f,0.0f); glVertex3f(-1.0f,0.0f,-14.0f);
		glTexCoord2f(1.0f,0.0f); glVertex3f(1.0f,0.0f,-14.0f);
		glTexCoord2f(1.0f,7.0f); glVertex3f(1.0f,0.0f,14.0f);
		glTexCoord2f(0.0f,7.0f); glVertex3f(-1.0f,0.0f,14.0f);

		glTexCoord2f(7.0f, 0.0f); glVertex3f(-1.0f,0.0f,14.0f);
		glTexCoord2f(7.0f, 1.0f); glVertex3f(-1.0f,2.0f,14.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,2.0f,-14.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,0.0f,-14.0f);
	glEnd();
}
